#!/bin/bash

yellow_echo() {
    echo -e "\e[33m$@\e[0m"
}

copy_file() {
    source_path=$1
    target_path=$2
    # 检查源文件或目录是否存在
    if [ ! -e "$source_path" ]; then
        echo "$source_path not exist, no copy file."
        return
    fi
    # 确保目标路径以 / 结尾，如果不是目录则取其目录部分
    local last_char=$(echo "$target_path" | awk '{sub(/ *$/,""); print substr($0,length)}')
    if [ "$last_char" == "/" ]; then
        target_dir=$target_path
    else
        if [ -d "$target_path" ]; then
            target_dir=$target_path
        else
            target_dir=$(dirname "$target_path")
        fi
    fi
    echo "target_dir: $target_dir"
    
    # 检查目标目录是否存在，如果不存在则创建
    if [ ! -d "$target_dir" ]; then
        mkdir -p "$target_dir"
        echo "$target_dir not exist, have created the directory."
    fi
    
    # 执行mv操作
    cp -r $source_path $target_path
    echo $source_path copy to $target_path
}

# 生成initrd
update_initrd() {
    VKERNEL=$(uname -r)
    # Array of available initrd creation commands
    commands="update-initramfs dracut mkinitrd"

    for cmd in $commands; do
        if [ -n "$cmd" ] && command -v "$cmd" >/dev/null 2>&1; then
            case "$cmd" in
                update-initramfs)
                    "$cmd" -u -k "$VKERNEL"
                    ;;
                dracut)
                    "$cmd" -f "/boot/initrd.img-$VKERNEL" "$VKERNEL"
                    ;;
                *)
                    # For any other initrd tool
                    "$cmd" -f "/boot/initrd.img-$VKERNEL" "$VKERNEL"
                    ;;
            esac
            return
        fi
    done

    # If none of the commands are found, print a message
    echo "No suitable initrd creation command found. Initrd not updated."
}

# 在用户的session中执行脚本
function run-in-user-session() {
    _curuser=$1
    _command="$2"
    _display_id=$(echo $_curuser | awk '{print $5}' | awk -F'(' '{print $2}' | awk -F')' '{print $1}')
    _username=$(echo $_curuser | awk '{print $1}')
    _user_id=$(id -u "$_username")
    echo $_display_id $_username $_user_id
    _environment=("DISPLAY=$_display_id" "DBUS_SESSION_BUS_ADDRESS=unix:path=/run/user/$_user_id/bus")
    if sudo -n true 2> /dev/null; then
        sudo -Hu "$_username" env "${_environment[@]}"  sh -c "$_command"
    else
        su "$_username" env "${_environment[@]}"  sh -c "$_command"
    fi
}

# 获取所有登陆图形界面的用户
curusers=$(who | grep -E 'tty[7-9]|tty[1-9][0-9]')
while read -r curuser; do
    _username=$(echo $_curuser | awk '{print $1}')
    # 切换GTK主题
    # 使用命令替换和$()来捕获run-in-user-session的执行结果和状态码
    if run-in-user-session "$curuser" "gsettings set org.cartoon.interface gtk-theme 'Cartoons'" &> /dev/null; then
        echo "$_username gtk-theme 主题切换成功"
    else
        yellow_echo "$_username gtk-theme 主题切换失败"
        yellow_echo "可尝试重启电脑或者注销用户，使其生效。"
        # 可选：打印错误信息
        echo "Error: $? (Details may be available in previous output)"
    fi

    # 切换ICON主题
    # 使用命令替换和$()来捕获run-in-user-session的执行结果和状态码
    if run-in-user-session "$curuser" "gsettings set org.cartoon.interface icon-theme 'Flat-Remix-Blue'" &> /dev/null; then
        echo "$_username icon-theme 主题切换成功"
    else
        yellow_echo "$_username icon-theme 主题切换失败"
        yellow_echo "可尝试重启电脑或者注销用户，使其生效。"
        # 可选：打印错误信息
        echo "Error: $? (Details may be available in previous output)"
    fi

    # 切换marco主题
    # 使用命令替换和$()来捕获run-in-user-session的执行结果和状态码
    if run-in-user-session "$curuser" "gsettings set org.cartoon.Marco.general theme 'Cartoons'" &> /dev/null; then
        echo "$_username Marco 主题切换成功"
    else
        yellow_echo "$_username Marco 主题切换失败"
        yellow_echo "可尝试重启电脑或者注销用户，使其生效。"
        # 可选：打印错误信息
        echo "Error: $? (Details may be available in previous output)"
    fi

    # 切换cartoon-panel样式
    # 使用命令替换和$()来捕获run-in-user-session的执行结果和状态码
    if run-in-user-session "$curuser" "gsettings set org.cartoon.panel default-layout 'default'" &> /dev/null; then
        echo "$_username panel 主题切换成功"
    else
        yellow_echo "$_username panel 主题切换失败"
        yellow_echo "可尝试重启电脑或者注销用户，使其生效。"
        # 可选：打印错误信息
        echo "Error: $? (Details may be available in previous output)"
    fi

    # 重启cartoon-panel
    # 使用括号执行命令并捕获其退出状态，避免直接使用!可能导致的逻辑混乱
    if (run-in-user-session "$curuser" "cartoon-panel --reset" &> /dev/null); then
        echo "$_username cartoon-panel 重启成功"
    else
        errorCode=$?
        yellow_echo "$_username cartoon-panel 重启失败"
        yellow_echo "可尝试重启电脑或者注销用户，使其生效。"
        if [ $errorCode -ne 0 ]; then
            echo "Error: $errorCode (Details may be available in system logs or previous output)"
        fi
    fi
done <<< "$curusers"

export PATH=$PATH:/sbin

# 修改默认主题使其生效
glib-compile-schemas /usr/share/glib-2.0/schemas/

# 切换plymouth主题
plymouth-set-default-theme loongnix
# Array of available initrd creation commands
update_initrd
